@@ -3,9 +3,9 @@
 ## Author: Brice Ozenne
 ## Created: jan 30 2018 (14:33) 
 ## Version: 
-## Last-Updated: mar 15 2018 (12:21) 
+## Last-Updated: apr 17 2018 (10:31) 
 ##           By: Brice Ozenne
-##     Update #: 312
+##     Update #: 328
 ##----------------------------------------------------------------------
 ## 
 ### Commentary: 
@@ -22,6 +22,8 @@
 #' @name compare2
 #'
 #' @param object an object that inherits from lm/gls/lme/lvmfit.
+#' @param df [logical] should the degree of freedoms of the Wald statistic be computed using the Satterthwaite correction?
+#' Otherwise the degree of freedoms are set to \code{Inf}, i.e. a normal distribution is used instead of a Student's t distribution when computing the p-values.
 #' @param bias.correct [logical] should the standard errors of the coefficients be corrected for small sample bias? Argument passed to \code{sCorrect}.
 #' @param cluster [integer vector] the grouping variable relative to which the observations are iid.
 #' Only required for \code{gls} models without correlation structure.
@@ -31,6 +33,7 @@
 #' @param robust [logical] should the robust standard errors be used instead of the model based standard errors?
 #' @param null [vector] the right hand side of the linear hypotheses to be tested.
 #' @param as.lava [logical] should the output be similar to the one return by \code{lava::compare}?
+#' @param F.test [logical] should a joint test be performed?
 #' @param level [numeric 0-1] the confidence level of the confidence interval.
 #' @param ...  [internal] only used by the generic method.
 #'
@@ -100,16 +103,16 @@
 ## * compare2.lm
 #' @rdname compare2
 #' @export
-compare2.lm <- function(object, bias.correct = TRUE, ...){
-    sCorrect(object) <- bias.correct
+compare2.lm <- function(object, df = TRUE, bias.correct = TRUE, ...){
+    sCorrect(object, df = df) <- bias.correct
     return(.compare2(object, ...))
 }
 
 ## * compare2.gls
 #' @rdname compare2
 #' @export
-compare2.gls <- function(object, bias.correct = TRUE, cluster = NULL, ...){
-    sCorrect(object, cluster = cluster) <- bias.correct
+compare2.gls <- function(object, df = TRUE, bias.correct = TRUE, cluster = NULL, ...){
+    sCorrect(object, df = df, cluster = cluster) <- bias.correct
     return(.compare2(object, ...))
 }
 
@@ -154,12 +157,16 @@ compare2.lvmfit2 <- function(object, ...){
 ## * .compare2
 #' @rdname compare2
 .compare2 <- function(object, par = NULL, contrast = NULL, null = NULL,
-                      robust = FALSE,
-                      as.lava = TRUE, level = 0.95){
+                      robust = FALSE, df = TRUE,
+                      as.lava = TRUE, F.test = TRUE, level = 0.95){
 
     ## ** extract information
-    dVcov.param <- object$sCorrect$dVcov.param
-
+    if(df){
+        dVcov.param <- object$sCorrect$dVcov.param
+    }else{
+        dVcov.param <- NULL
+    }
+    
     param <- object$sCorrect$param
     if(robust){
         vcov.param <- crossprod(iid2(object))
@@ -192,12 +199,24 @@ compare2.lvmfit2 <- function(object, ...){
             stop("Argument \'contrast\' and argument \'par\' cannot be both NULL \n",
                  "Please specify the null hypotheses using one of the two arguments \n")
         }
+        if(is.null(colnames(contrast))){
+            stop("Argument \'contrast\' must have column names \n")
+        }
+        if(any(colnames(contrast) %in% name.param == FALSE)){
+            txt <- setdiff(colnames(contrast), name.param)
+            stop("Argument \'contrast\' has incorrect column names \n",
+                 "invalid name(s): \"",paste(txt, collapse = "\" \""),"\"\n")
+        }
+        if(any(name.param %in% colnames(contrast) == FALSE)){
+            txt <- setdiff(name.param, colnames(contrast))
+            stop("Argument \'contrast\' has incorrect column names \n",
+                 "missing name(s): \"",paste(txt, collapse = "\" \""),"\"\n")
+        }
         if(NCOL(contrast) != n.param){
             stop("Argument \'contrast\' should be a matrix with ",n.param," columns \n")
         }
-        if(is.null(colnames(contrast)) || any(colnames(contrast) != name.param)){
-            stop("Argument \'contrast\' has incorrect column names \n")
-        }
+        ## reorder columns according to coefficients
+        contrast <- contrast[,name.param,drop=FALSE]
         if(any(abs(svd(contrast)$d)<1e-10)){
             stop("Argument \'contrast\' is singular \n")
         }
@@ -274,20 +293,23 @@ compare2.lvmfit2 <- function(object, ...){
     df.table$`p-value` <- as.numeric(2*(1-stats::pt(abs(df.table$statistic), df = df.table$df)))
     
 ### ** multivariate F test
-    ## statistic
-    stat.F <- try(t(C.p) %*% solve(C.vcov.C)%*% (C.p) / n.hypo, silent = TRUE)
-     
-    ## store
     df.table <- rbind(df.table, global = rep(NA,5))
-    if(!inherits(stat.F,"try-error")){
-        df.table["global", "statistic"] <- as.numeric(stat.F)
-        df.table["global", "df"] <- df.F
-        df.table["global", "p-value"] <- 1 - stats::pf(df.table["global", "statistic"],
-                                                       df1 = n.hypo,
-                                                       df2 = df.table["global", "df"])
-        error <- NULL
-    }else{
-        error <- df.table
+    error <- NULL
+     
+    if(F.test){
+        ## statistic
+        stat.F <- try(t(C.p) %*% solve(C.vcov.C)%*% (C.p) / n.hypo, silent = TRUE)
+     
+        ## store
+        if(!inherits(stat.F,"try-error")){
+            df.table["global", "statistic"] <- as.numeric(stat.F)
+            df.table["global", "df"] <- df.F
+            df.table["global", "p-value"] <- 1 - stats::pf(df.table["global", "statistic"],
+                                                           df1 = n.hypo,
+                                                           df2 = df.table["global", "df"])
+        }else{
+            error <- df.table
+        }
     }
     
     ## ** export
