@@ -3,9 +3,9 @@
 ## Author: Brice Ozenne
 ## Created: jan  3 2018 (14:29) 
 ## Version: 
-## Last-Updated: mar 15 2018 (18:15) 
+## Last-Updated: apr 10 2018 (14:28) 
 ##           By: Brice Ozenne
-##     Update #: 931
+##     Update #: 1269
 ##----------------------------------------------------------------------
 ## 
 ### Commentary: 
@@ -26,7 +26,8 @@
 #' @param cluster [integer vector] the grouping variable relative to which the observations are iid.
 #' Only required for \code{gls} models with no correlation argument.
 #' @param value [logical] value for the arguments \code{adjust.Omega} and \code{adjust.n}.
-#' @param df [logical] should the first derivative of the expected information matrix be computed. Required when computing the degrees of freedom of the test statistics.
+#' @param df [logical] should the degree of freedoms of the Wald statistic be computed using the Satterthwaite correction?
+#' Otherwise the degree of freedoms are set to \code{Inf}, i.e. a normal distribution is used instead of a Student's t distribution when computing the p-values.
 #' @param adjust.Omega [logical] should the standard errors of the coefficients be corrected for small sample bias?
 #' @param adjust.n [logical] should the correction for the degree of freedom be performed?
 #' @param tol [numeric >0] the minimum absolute difference between two estimation of the small sample bias.
@@ -38,6 +39,8 @@
 #' @param score [internal] export the score.
 #' @param ... [internal] only used by the generic method or by the <- methods.
 #'
+#' @details The argument \code{value} is equivalent to the argument \code{bias.correct} of the function \code{summary2}.
+#' 
 #' @concept small sample inference
 #' @concept derivative of the score equation
 #' n <- 5e1
@@ -49,23 +52,23 @@
 #' m <- lvm(formula.lvm)
 #' distribution(m,~Id) <- sequence.lvm(0)
 #' set.seed(10)
-#' d <- sim(m,n)
+#' d <- lava::sim(m,n)
 #'
 #' ## linear model
 #' e.lm <- lm(formula.lvm,data=d)
 #' system.time(
-#' sCorrect(e.lm) <- TRUE
+#' sCorrect(e.lm) <- TRUE ## i.e. bias.correct = TRUE
 #')
 #' 
 #' ## gls model
 #' library(nlme)
 #' e.gls <- gls(formula.lvm, data = d, method = "ML")
-#' sCorrect(e.gls, cluster = 1:NROW(d)) <- TRUE
+#' sCorrect(e.gls, cluster = 1:NROW(d)) <- TRUE ## i.e. bias.correct = TRUE
 #' summary2(e.gls)
 #'
 #' ## latent variable model
 #' e.lvm <- estimate(lvm(formula.lvm),data=d)
-#' sCorrect(e.lvm) <- TRUE
+#' sCorrect(e.lvm) <- TRUE ## i.e. bias.correct = TRUE
 #' summary2(e.lvm)
 #' 
 #' @export
@@ -83,7 +86,6 @@ sCorrect.lm <- function(object, adjust.Omega = TRUE, adjust.n = TRUE,
     
 ### ** Extract quantities from object
     name.endogenous <- all.vars(stats::update(formula(object), ".~1"))
-    n.cluster <- stats::nobs(object) + length(object$na.action)
 
     if(is.null(param)){
         param <- .coef2(object)
@@ -102,12 +104,31 @@ sCorrect.lm <- function(object, adjust.Omega = TRUE, adjust.n = TRUE,
     name.meanparam <- attr(model.param,"mean.coef")
     name.varparam <- attr(model.param,"var.coef")
 
+    if(is.null(data)){
+      data <- model.frame(object)
+    }
+
+### ** Number of samples
+    test.NNA <- sum(is.na(data[[name.endogenous]]))==0
+    if(any(test.NNA==FALSE)){ ## complete case analysis
+        if(trace>0){
+            cat("* Exclude missing values and recompute moments and residuals ")
+        }        
+        data <- data[which(test.NNA),,drop=FALSE]
+        if(trace>0){
+            cat("- done \n")
+        }        
+    }
+    
+    n.cluster <- NROW(data)
+    
 ### ** Compute conditional moments
     if(trace>0){
         cat("Compute conditional moments")
     }
-    dMoments <- conditionalMoment(object, name.endogenous = name.endogenous,
-                                  second.order = df)
+    object$conditionalMoment <- conditionalMoment(object, data = data, param = model.param,
+                                                  name.endogenous = name.endogenous,
+                                                  first.order = TRUE, second.order = FALSE)
     if(trace>0){
         cat(" - done \n")
     }
@@ -116,36 +137,26 @@ sCorrect.lm <- function(object, adjust.Omega = TRUE, adjust.n = TRUE,
     if(trace>0){
         cat("* Extract residuals ")
     }
-    if(is.null(data)){
-        X <- model.matrix(object)
-    }else{
-        X <- model.matrix(formula(object), data)
-    }
-    Y <- object$residuals + object$fitted.values
-    object.residuals <- Y - X %*% cbind(model.param[attr(model.param,"mean.coef")])    
+    object.residuals <- data[[name.endogenous]] - object$conditionalMoment$mu    
     dimnames(object.residuals) <- list(NULL, name.endogenous)
     if(trace>0){
         cat("- done \n")
     }
-    
-    ## ** Compute residual variance
-    if(trace>0){
-        cat("* Reconstruct estimated residual variance ")
-    }
-    Omega <- matrix(param["sigma2"], nrow = 1, ncol = 1,
-                    dimnames = list(name.endogenous, name.endogenous))
-    if(trace>0){
-        cat("- done \n")
-    }
 
-    ## ** args
+### ** args
     args <- list(adjust.Omega = adjust.Omega,
                  adjust.n = adjust.n,
                  df = df,
                  numeric.derivative = numeric.derivative,
                  tol = tol, n.iter = n.iter)
+
+    if(df && numeric.derivative){
+        argsNumDeriv <- list(data=data)
+    }else{
+        argsNumDeriv <- list()
+    }
     
-    ## ** correction
+### ** correction
     if(df == FALSE){
         derivative <- "none"
     }else if(numeric.derivative){
@@ -155,28 +166,16 @@ sCorrect.lm <- function(object, adjust.Omega = TRUE, adjust.n = TRUE,
     }
 
     out <- .sCorrect(object,
-                     data = data,
                      param = model.param,
                      epsilon = object.residuals,
-                     Omega = Omega,
-                     dmu = dMoments$dmu,
-                     dOmega = dMoments$dOmega,
-                     d2mu = NULL,
-                     d2Omega = NULL,
                      name.param = name.param,
-                     name.meanparam = name.meanparam,
-                     name.varparam = name.varparam,
                      name.endogenous = name.endogenous,
-                     name.3deriv = dMoments$name.3deriv,
                      n.cluster = n.cluster,
                      index.Omega = NULL,
-                     adjust.Omega = adjust.Omega,
-                     adjust.n = adjust.n,
-                     tol = tol,
-                     n.iter = n.iter,
                      score = score,
                      derivative = derivative,
                      args = args,
+                     argsNumDeriv = argsNumDeriv,
                      trace = trace,
                      ...)
     
@@ -184,6 +183,13 @@ sCorrect.lm <- function(object, adjust.Omega = TRUE, adjust.n = TRUE,
     return(out)    
 }
 
+## * sCorrect.lm2
+#' @rdname sCorrect
+#' @export
+sCorrect.lm2 <- function(object, ...){
+    class(object) <- setdiff(class(object),"lm2")
+    return(sCorrect(object, ...))    
+}
 ## * sCorrect.gls
 #' @rdname sCorrect
 #' @export
@@ -192,7 +198,6 @@ sCorrect.gls <- function(object, cluster, adjust.Omega = TRUE, adjust.n = TRUE,
                          param = NULL, data = NULL,
                          tol = 1e-5, n.iter = 20, trace = 0,
                          ...){
-
 ### ** limitations
     if(object$method!="ML"){
         if(adjust.Omega==TRUE || adjust.n == TRUE){
@@ -204,6 +209,7 @@ sCorrect.gls <- function(object, cluster, adjust.Omega = TRUE, adjust.n = TRUE,
     
     ## check valid class for corStruct and varStruct: see .getVarCov2
 ### ** Extract quantities from the model
+
     ## *** data
     if(is.null(data)){
         data <- extractData(object, design.matrix = FALSE, as.data.frame = TRUE,
@@ -213,7 +219,6 @@ sCorrect.gls <- function(object, cluster, adjust.Omega = TRUE, adjust.n = TRUE,
     ## *** endogenous variable
     formula.object <- .getFormula2(object)
     name.Y <- all.vars(stats::update(formula.object, ".~1"))
-    Y <- data[[name.Y]]
     
     ## *** parameters
     model.param <- .coef2(object)
@@ -238,99 +243,109 @@ sCorrect.gls <- function(object, cluster, adjust.Omega = TRUE, adjust.n = TRUE,
     res.cluster <- .getCluster2(object,
                                 data = data,
                                 cluster = cluster)
+    n.cluster <- res.cluster$n.cluster
+    cluster <- res.cluster$cluster
     if(trace>0){
         cat("- done \n")
     }
+
+    ## data before re-ordering
+    args <- list(adjust.Omega = adjust.Omega,
+                 adjust.n = adjust.n,                     
+                 df = df,
+                 numeric.derivative = numeric.derivative,
+                 tol = tol, n.iter = n.iter,
+                 cluster = cluster) ## for score2
+
+    if(df && numeric.derivative){
+        argsNumDeriv <- list(data = data)
+    }else{
+        argsNumDeriv <- list()
+    }
     
     ## *** repetition relative to each observation
     if(trace>0){
         cat("* Relate observations to endogenous variables ")
     }
-
     res.index <- .getIndexOmega2(object,
                                  param = model.param,
                                  attr.param = attributes(model.param),
                                  name.Y = name.Y,
-                                 cluster = res.cluster$cluster,
+                                 cluster = cluster,
                                  levels.cluster = res.cluster$levels.cluster,
                                  data = data)
-    index.Omega <- res.index$index.Omega
-
-    if(trace>0){
-        cat("- done \n")
-    }
-
-    ### ** Reconstruct residuals variance covariance matrix
-    if(trace>0){
-        cat("* Reconstruct estimated residual variance-covariance matrix ")
-    }
-    
-    Omega <- .getVarCov2(object,
-                         param = model.param,
-                         attr.param = attributes(model.param),
-                         name.endogenous = res.index$name.endogenous,
-                         n.endogenous = res.index$n.endogenous,
-                         ref.group = res.index$ref.group)
-
-    if(trace>0){
-        cat("- done \n")
-    }
-    cluster <- res.cluster$cluster
-    n.cluster <- res.cluster$n.cluster
     name.endogenous <- res.index$name.endogenous
     n.endogenous <- res.index$n.endogenous
+    index.Omega <- res.index$index.Omega
+    if(trace>0){
+        cat("- done \n")
+    }
+  
+    ## *** sort data by group
+    vec.endogenous <- rep(NA, length(cluster))
+    for(iC in 1:n.cluster){
+        vec.endogenous[cluster==iC] <- index.Omega[[res.cluster$levels.cluster[iC]]]
+    }
+    order.obs <- order(cluster,vec.endogenous)
+    if(is.unsorted(order.obs)==TRUE){
+        data <- data[order.obs,,drop=FALSE]
+        cluster <- cluster[order.obs]
+        vec.endogenous <- vec.endogenous[order.obs]
+    }
+    test.reorder <- is.unsorted(res.cluster$levels.cluster)
+    if(test.reorder==TRUE){
+        index.Omega <- index.Omega[res.cluster$levels.cluster]
+    }
+    ## for vector format to matrix format (for residuals and fitted values)
+    vec.OmegaMat <- cluster + (vec.endogenous-1)*n.cluster
+    
+    ## M.check <- matrix(NA, nrow = n.cluster, ncol = n.endogenous)
+    ## M.check[vec.endogenous + (cluster-1)*n.endogenous] <- data[["G"]]
+    ## M.check[cluster + (vec.endogenous-1)*n.cluster] <- data[["G"]]
 
 ### ** Compute conditional moments and derivatives
     if(trace>0){
         cat("* Compute conditional moments ")
     }
-    dMoments <- conditionalMoment(object,
-                                  data = data,
-                                  formula = formula.object,
-                                  param = model.param,
-                                  attr.param = attributes(model.param)[-1],
-                                  ref.group = res.index$ref.group,
-                                  second.order = df,
-                                  n.cluster = n.cluster,
-                                  cluster = cluster,
-                                  name.endogenous = name.endogenous,
-                                  index.Omega = index.Omega)
+    object$conditionalMoment <- conditionalMoment(object,
+                                                  data = data,
+                                                  formula = formula.object,
+                                                  param = model.param,
+                                                  attr.param = attributes(model.param)[-1],
+                                                  ref.group = res.index$ref.group,
+                                                  first.order = TRUE,
+                                                  second.order = FALSE,
+                                                  n.cluster = n.cluster,
+                                                  cluster = cluster,
+                                                  name.endogenous = name.endogenous,
+                                                  n.endogenous = n.endogenous,
+                                                  index.Omega = index.Omega,
+                                                  vec.OmegaMat = vec.OmegaMat)
     if(trace>0){
         cat("- done \n")
     }
-    
+   
 ### ** Compute observed residuals
     if(trace>0){
         cat("* Extract residuals ")
     }
     epsilon <- matrix(NA, nrow = n.cluster, ncol = n.endogenous,
                       dimnames = list(NULL, name.endogenous))
-    for(iC in 1:n.cluster){
-        iIndex <- which(cluster == iC)
-        epsilon[iC,index.Omega[[iC]]] <- as.double(Y[iIndex] - dMoments$X[iIndex,,drop=FALSE] %*% model.param[name.meanparam])
-    }
-
+    epsilon[vec.OmegaMat] <- data[[name.Y]]
+    epsilon <- epsilon -  object$conditionalMoment$mu
+    
     if(trace>0){
         cat("- done \n")
     }
-    ## stats::residuals(object)-as.double(t(epsilon))
-
+    ## stats::residuals(object)-na.omit(as.double(t(epsilon)))
+    ## epsilon + object$conditionalMoment$mu
+    ## data
+    ## as.double(stats::residuals(object))
     ## ** Check missing value
     if(all(!is.na(epsilon))){
         index.Omega <- NULL
     }
     
-    ## ** param with non-zero third derivative
-    name.3deriv <- name.varparam
-
-    ## ** args
-    args <- list(adjust.Omega = adjust.Omega,
-                 adjust.n = adjust.n,                     
-                 df = df,
-                 cluster = cluster,
-                 numeric.derivative = numeric.derivative,
-                 tol = tol, n.iter = n.iter)
-    
     ## ** correction
     if(df == FALSE){
         derivative <- "none"
@@ -340,40 +355,49 @@ sCorrect.gls <- function(object, cluster, adjust.Omega = TRUE, adjust.n = TRUE,
         derivative <- "analytic"
     }
     out <- .sCorrect(object,
-                     data = data,
                      param = model.param,
                      epsilon = epsilon,
-                     Omega = Omega,
-                     dmu = dMoments$dmu,
-                     dOmega = dMoments$dOmega,
-                     d2mu = NULL,
-                     d2Omega = dMoments$d2Omega,
                      name.param = name.param,
-                     name.meanparam = name.meanparam,
-                     name.varparam = name.varparam,
                      name.endogenous = name.endogenous,
-                     name.3deriv = name.3deriv,
                      n.cluster = n.cluster,
                      index.Omega = index.Omega,
-                     adjust.Omega = adjust.Omega,
-                     adjust.n = adjust.n,
-                     tol = tol,
-                     n.iter = n.iter,
                      score = score,
                      derivative = derivative,
                      args = args,
+                     argsNumDeriv = argsNumDeriv,
                      trace = trace,
                      ...)
     
-    ## ** export    
+    ## ** export
+    ## *** restaure original order
+    if(test.reorder==TRUE){
+        out$score <- out$score[res.cluster$levels.cluster,,drop=FALSE]
+        out$residuals <- out$residuals[res.cluster$levels.cluster,,drop=FALSE]
+        out$leverage <- out$leverage[res.cluster$levels.cluster,,drop=FALSE]
+    }    
+    ##
     return(out)          
  
 }
 
+## * sCorrect.gls2
+#' @rdname sCorrect
+#' @export
+sCorrect.gls2 <- function(object, ...){
+    class(object) <- setdiff(class(object),"gls2")
+    return(sCorrect(object, ...))    
+}
 ## * sCorrect.lme
 #' @rdname sCorrect
 #' @export
 sCorrect.lme <- sCorrect.gls
+## * sCorrect.lme2
+#' @rdname sCorrect
+#' @export
+sCorrect.lme2 <- function(object, ...){
+    class(object) <- setdiff(class(object),"lme2")
+    return(sCorrect(object, ...))    
+}
 
 ## * sCorrect.lvmfit
 #' @rdname sCorrect
@@ -384,10 +408,13 @@ sCorrect.lvmfit <- function(object, adjust.Omega = TRUE, adjust.n = TRUE,
                             tol = 1e-5, n.iter = 20, trace = 0,
                             ...){
 
-    ## ** Check valid lvm object
+### ** Check valid lvm object
     if("multigroupfit" %in% class(object)){
         stop("sCorrect does not handle multigroup models \n")
     }
+    if(!is.null(object$cluster)){
+        stop("sCorrect does not handle lvmfit object with cluster \n")
+    }
    
     if(length(object$model$attributes$ordinal)>0){
         name.t <- names(object$model$attributes$ordinal)
@@ -401,7 +428,7 @@ sCorrect.lvmfit <- function(object, adjust.Omega = TRUE, adjust.n = TRUE,
              "transformed variable(s): \"",paste(name.t, collapse = "\" \""),"\"\n")
     }
     
-    ## ** Extract quantities from object
+### ** Extract quantities from object
     name.endogenous <- endogenous(object)
 
     model.param <- lava::pars(object)
@@ -420,9 +447,10 @@ sCorrect.lvmfit <- function(object, adjust.Omega = TRUE, adjust.n = TRUE,
 
     name.param <- names(model.param)
 
-    n.latent <- length(latent(object))
+    name.latent <- latent(object)
+    n.latent <- length(name.latent)
 
-    ### ** number of samples
+### ** number of samples
     test.NNA <- rowSums(is.na(data[,name.endogenous,drop=FALSE]))==0    
     if(any(test.NNA==FALSE) && !inherits(object,"lvm.missing")){ ## complete case analysis
         if(trace>0){
@@ -436,70 +464,59 @@ sCorrect.lvmfit <- function(object, adjust.Omega = TRUE, adjust.n = TRUE,
     
     n.cluster <- NROW(data)
 
-    ### ** Compute conditional moments and derivatives
+### ** Compute conditional moments and derivatives
     if(trace>0){
-        cat("* Compute conditional moments ")
+        cat("* Compute conditional moments and their derivative ")
+    }
+    object$conditionalMoment <- conditionalMoment(object, data = data, param = model.param,
+                                                  first.order = TRUE, second.order = FALSE, usefit = TRUE)
+    if(df == TRUE && (numeric.derivative == FALSE)){
+        object$conditionalMoment$d2Moment.init <- skeletonDtheta2(lava::Model(object),
+                                                                  data = data,
+                                                                  df.param.all = object$conditionalMoment$df.param,
+                                                                  param2originalLink = object$conditionalMoment$param2originalLink,
+                                                                  name.latent = name.latent)
     }
-    dMoments <- conditionalMoment(object, data = data, param = model.param,
-                                  second.order = df, usefit = TRUE)
     if(trace>0){
         cat("- done \n")
     }
 
-    name.meanparam <- names(dMoments$dmu)
-    name.varparam <- names(dMoments$dOmega)
-
-    #### ** Compute residuals
+#### ** Compute residuals
     if(trace>0){
         cat("* Extract residuals ")
     }
-    epsilon <- .calcResidualsLVM(data = data, dMoments = dMoments,
-                                 n.latent = n.latent,
-                                 name.endogenous = name.endogenous)
+    epsilon <- as.matrix(data[, name.endogenous,drop=FALSE] - object$conditionalMoment$mu)
+    ## residuals(object) - epsilon
     if(trace>0){
         cat("- done \n")
     }
 
 ### ** Identify missing values
     if(any(test.NNA==FALSE) && inherits(object,"lvm.missing")){ ## full information
-            if(trace>0){
-                cat("* Identify missing values ")
-            }
-            index.Omega <- lapply(1:n.cluster,function(iC){which(!is.na(epsilon[iC,]))})
-            if(trace>0){
-                cat("- done \n")
-            }        
+        if(trace>0){
+            cat("* Identify missing values ")
+        }
+        index.Omega <- lapply(1:n.cluster,function(iC){which(!is.na(epsilon[iC,]))})
+        if(trace>0){
+            cat("- done \n")
+        }        
     }else{
         index.Omega <- NULL
     }
 
-    ### ** Compute residual variance covariance matrix
-    if(trace>0){
-        cat("* Reconstruct residual variance-covariance matrix ")
-    }
-
-    Omega <- .calcOmegaLVM(dMoments, n.latent = n.latent)
-
-    if(trace>0){
-        cat("- done \n")
-    }
-
-    ## ** param with non-zero third derivative
-    type.3deriv <- c("alpha","Gamma","Lambda","B","Psi_var","Sigma_var","Psi_cov","Sigma_cov")
-    index.keep <- intersect(which(!is.na(dMoments$df.param$lava)),
-                            which(dMoments$df.param$detail %in% type.3deriv)
-                            )
-    
-    name.3deriv <- dMoments$df.param[index.keep, "originalLink"]
-
-    ## ** args
+### ** args
     args <- list(adjust.Omega = adjust.Omega,
                  adjust.n = adjust.n,                     
                  df = df,
                  numeric.derivative = numeric.derivative,
                  tol = tol, n.iter = n.iter)
-
-    ## ** correction
+    if(df && numeric.derivative){
+        argsNumDeriv <- list(data = data)
+    }else{
+        argsNumDeriv <- list()
+    }
+    
+### ** correction
     if(df == FALSE){
         derivative <- "none"
     }else if(numeric.derivative){
@@ -508,29 +525,17 @@ sCorrect.lvmfit <- function(object, adjust.Omega = TRUE, adjust.n = TRUE,
         derivative <- "analytic"
     }
 
-   out <- .sCorrect(object,
-                     data = data,
+    out <- .sCorrect(object,
                      param = model.param,
                      epsilon = epsilon,
-                     Omega = Omega,
-                     dmu = dMoments$dmu,
-                     dOmega = dMoments$dOmega,
-                     d2mu = dMoments$d2mu,
-                     d2Omega = dMoments$d2Omega,
                      name.param = name.param,
-                     name.meanparam = name.meanparam,
-                     name.varparam = name.varparam,
                      name.endogenous = name.endogenous,
-                     name.3deriv = name.3deriv,
                      n.cluster = n.cluster,
                      index.Omega = index.Omega,
-                     adjust.Omega = adjust.Omega,
-                     adjust.n = adjust.n,
-                     tol = tol,
-                     n.iter = n.iter,
                      score = score,
                      derivative = derivative,
                      args = args,
+                     argsNumDeriv = argsNumDeriv,
                      trace = trace,
                      ...)
 
@@ -546,10 +551,10 @@ sCorrect.lvmfit2 <- function(object, ...){
     return(sCorrect(object, ...))    
 }
 ## * .sCorrect
-.sCorrect <- function(object, data, param, epsilon, Omega, dmu, dOmega, d2mu, d2Omega, 
-                      name.param, name.meanparam, name.varparam, name.endogenous, name.3deriv,
+.sCorrect <- function(object, param, epsilon, 
+                      name.param, name.endogenous, 
                       n.cluster, index.Omega,
-                      adjust.Omega, adjust.n, tol, n.iter, score, derivative, args, trace){
+                      score, derivative, args, argsNumDeriv, trace){
 
     n.param <- length(param)
     if(!is.null(index.Omega)){
@@ -557,88 +562,94 @@ sCorrect.lvmfit2 <- function(object, ...){
     }else{
         n.endogenous.cluster <- NULL
     }
-    ## ** order param names
+    name.3deriv <- object$conditionalMoment$name.3deriv
+    
+    ## ** check names of the mean and variance parameters
+    name.meanparam <- names(object$conditionalMoment$dmu)
     name.meanparam <- as.character(sort(factor(name.meanparam, levels = name.param)))
     if(any(is.na(name.meanparam))){
         stop("An element in name.meanparam is not in name.param. \n")
     }
-    if(length(name.meanparam)>0 && !identical(sort(name.meanparam),sort(names(dmu)))){
+    if(length(name.meanparam)>0 && !identical(sort(name.meanparam),sort(names(object$conditionalMoment$dmu)))){
         stop("Mismatch first derivative of the conditional mean and name.meanparam \n")
     }
-    dmu <- dmu[name.meanparam]    
 
+    name.varparam <- names(object$conditionalMoment$dOmega)
     name.varparam <- as.character(sort(factor(name.varparam, levels = name.param)))
     if(any(is.na(name.varparam))){
         stop("An element in name.varparam is not in name.param. \n")
     }
-    if(length(name.varparam)>0 && !identical(sort(name.varparam),sort(names(dOmega)))){
+    if(length(name.varparam)>0 && !identical(sort(name.varparam),sort(names(object$conditionalMoment$dOmega)))){
         stop("Mismatch first derivative of the conditional variance and name.varparam \n")
     }
-    dOmega <- dOmega[name.varparam]
-
+    if(length(name.varparam)==0){
+        args$adjust.n <- FALSE
+        args$adjust.Omega <- FALSE
+    }
+    
     ## ** corrected ML estimates
-    out  <- adjustEstimate(epsilon = epsilon,
-                           Omega = Omega,
-                           dmu = dmu,
-                           dOmega = dOmega,
-                           n.cluster = n.cluster,
-                           name.param = name.param,
-                           name.meanparam = name.meanparam,
-                           name.varparam = name.varparam,
-                           name.endogenous = name.endogenous,
-                           index.Omega = index.Omega, ## mode2
-                           adjust.Omega = adjust.Omega,
-                           adjust.n = adjust.n,
-                           tol = tol, n.iter = n.iter,
-                           trace = trace)    
-    out$param <- param
+    object  <- .estimate2(object = object,
+                          epsilon = epsilon,
+                          n.cluster = n.cluster,
+                          name.param = name.param,
+                          name.meanparam = name.meanparam,
+                          name.varparam = name.varparam,
+                          name.endogenous = name.endogenous,
+                          index.Omega = index.Omega, ## mode2
+                          adjust.Omega = args$adjust.Omega,
+                          adjust.n = args$adjust.n,
+                          tol = args$tol, n.iter = args$n.iter,
+                          trace = trace)
 
+    Omega <- object$conditionalMoment$Omega    
+    if(!is.null(index.Omega)){
+        OmegaM1 <- lapply(1:n.cluster, function(iC){
+            return(solve(Omega[index.Omega[[iC]],index.Omega[[iC]]]))
+        })    
+    }else{
+        OmegaM1 <- chol2inv(chol(Omega))
+    }
+    
     ## ** corrected score
     if(score){
         if(trace>0){
-            if(adjust.n == FALSE && adjust.Omega == FALSE){
+            if(args$adjust.n == FALSE && args$adjust.Omega == FALSE){
                 cat("* Compute score ")
             }else{
                 cat("* Compute corrected score ")
             }
         }
-        out$score <- .score2(epsilon = out$epsilon,
-                             Omega = out$Omega,
-                             OmegaM1 = out$OmegaM1,
-                             dmu = dmu,
-                             dOmega = dOmega,
-                             name.param = name.param,
-                             name.meanparam = name.meanparam,
-                             name.varparam = name.varparam,
-                             index.Omega = index.Omega, ## mode2
-                             n.cluster = n.cluster,
-                             indiv = TRUE)
+        object$dVcov$score <- .score2(epsilon = object$dVcov$residuals,
+                                      Omega = Omega,
+                                      OmegaM1 = OmegaM1,
+                                      dmu = object$conditionalMoment$dmu,
+                                      dOmega = object$conditionalMoment$dOmega,
+                                      name.param = name.param,
+                                      name.meanparam = name.meanparam,
+                                      name.varparam = name.varparam,
+                                      index.Omega = index.Omega, ## mode2
+                                      n.cluster = n.cluster,
+                                      indiv = TRUE)
         if(trace>0){
             cat("- done \n")
         }
     }
-    
-    ## ** first derivative of the expected information matrix
-    if(length(name.3deriv)==0){
-        out$dVcov.param <- NULL
+
+    ## ** first derivative of the expected information matrix    
+    if(args$df == FALSE || length(name.3deriv)==0){
+        object$dVcov$dVcov.param <- NULL
     }else if(derivative == "none"){
-        out$dVcov.param <- NA
+        object$dVcov$dVcov.param <- NA
     }else if(derivative == "numeric"){
         if(trace>0){
             cat("Compute first derivative of the information matrix using numerical differentiation ")
         }
-        if(adjust.Omega || adjust.n){
+        if(args$adjust.Omega || args$adjust.n){
             warning("The numerical derivative of the information matrix is computed ignoring the small sample correction \n")
         }
 
-        if("lvmfit" %in% class(object)){
-            object$conditionalMoment <- conditionalMoment(lava::Model(object), data = data,
-                                                          usefit = FALSE, second.order = FALSE,
-                                                          name.endogenous = endogenous(object),
-                                                          name.latent = latent(object))
-        }
         args.tempo <- args
-        args.tempo$data <- data
+        args.tempo$data <- argsNumDeriv$data
         args.tempo$df <- FALSE
         args.tempo$score <- FALSE
 
@@ -646,10 +657,10 @@ sCorrect.lvmfit2 <- function(object, ...){
         calcVcov <- function(iParam){ # x <- p.obj
             pp <- param
             pp[names(iParam)] <- iParam
-            
             vcov.param <- do.call(sCorrect,
                                   args = c(list(object, param = pp), args.tempo))$vcov.param
             return(vcov.param)
+            ## return(solve(vcov.param))
         }
 
         ### *** numerical derivative
@@ -661,35 +672,39 @@ sCorrect.lvmfit2 <- function(object, ...){
         jac.param <- param[name.3deriv]
         res.numDeriv <- numDeriv::jacobian(calcVcov, x = jac.param, method = "Richardson")
         
-        out$dVcov.param <- array(res.numDeriv,
-                                 dim = c(n.param,n.param,length(name.3deriv)),
-                                 dimnames = list(name.param, name.param, name.3deriv))
-         if(trace>0){
+        object$dVcov$dVcov.param <- array(res.numDeriv,
+                                          dim = c(n.param,n.param,length(name.3deriv)),
+                                          dimnames = list(name.param, name.param, name.3deriv))
+        if(trace>0){
             cat("- done \n")
-         }
+        }
         
     }else if(derivative == "analytic"){
         if(trace>0){
             cat("* Compute first derivative of the information matrix using analytic formula ")
         }
 
-        dInfo.dtheta <- .dInformation2(dmu = dmu,
-                                       d2mu = d2mu,
-                                       dOmega = dOmega,
-                                       d2Omega = d2Omega,
-                                       Omega = out$Omega,
-                                       OmegaM1 = out$OmegaM1,
-                                       n.corrected = out$n.corrected,
+        ## update conditional moments
+        resD2 <- skeletonDtheta2(object)
+
+        dInfo.dtheta <- .dInformation2(dmu = object$conditionalMoment$dmu,
+                                       d2mu = resD2$d2mu,
+                                       dOmega = object$conditionalMoment$dOmega,
+                                       d2Omega = resD2$d2Omega,
+                                       Omega = Omega,
+                                       OmegaM1 = OmegaM1,
+                                       n.corrected = object$dVcov$n.corrected,
                                        n.cluster = n.cluster,
                                        index.Omega = index.Omega,
-                                       leverage = out$leverage,
+                                       leverage = object$dVcov$leverage,
                                        name.param  = name.param,
                                        name.3deriv = name.3deriv)
         p3 <- dim(dInfo.dtheta)[3]
-        out$dVcov.param <- array(NA, dim = dim(dInfo.dtheta), dimnames = dimnames(dInfo.dtheta))
+        object$dVcov$dVcov.param <- array(NA, dim = dim(dInfo.dtheta), dimnames = dimnames(dInfo.dtheta))
         
         for(iP in 1:p3){
-            out$dVcov.param[,,iP] <- - out$vcov.param %*% dInfo.dtheta[,,iP] %*% out$vcov.param 
+            object$dVcov$dVcov.param[,,iP] <- - object$dVcov$vcov.param %*% dInfo.dtheta[,,iP] %*% object$dVcov$vcov.param
+            ## object$dVcov$dVcov.param[,,iP] <- dInfo.dtheta[,,iP]
         }
 
         if(trace>0){
@@ -698,8 +713,8 @@ sCorrect.lvmfit2 <- function(object, ...){
     }
        
     ## ** export
-    out$args <- args
-    return(out)
+    object$dVcov$args <- args
+    return(object$dVcov)
 }
 
 ## * sCorrect<-
@@ -714,7 +729,13 @@ sCorrect.lvmfit2 <- function(object, ...){
 `sCorrect<-.lm` <- function(x, ..., value){
     x$sCorrect <- sCorrect(x, ..., adjust.Omega = value, adjust.n = value)
     class(x) <- append("lm2",class(x))
-
+    return(x)
+}    
+## * sCorrect<-.lm2
+#' @rdname sCorrect
+#' @export
+`sCorrect<-.lm2` <- function(x, ..., value){
+    x$sCorrect <- sCorrect(x, ..., adjust.Omega = value, adjust.n = value)
     return(x)
 }    
 ## * sCorrect<-.gls
@@ -723,7 +744,13 @@ sCorrect.lvmfit2 <- function(object, ...){
 `sCorrect<-.gls` <- function(x, ..., value){
     x$sCorrect <- sCorrect(x, ..., adjust.Omega = value, adjust.n = value)
     class(x) <- append("gls2",class(x))
-
+    return(x)
+}    
+## * sCorrect<-.gls2
+#' @rdname sCorrect
+#' @export
+`sCorrect<-.gls2` <- function(x, ..., value){
+    x$sCorrect <- sCorrect(x, ..., adjust.Omega = value, adjust.n = value)
     return(x)
 }    
 ## * sCorrect<-.lme
@@ -732,21 +759,40 @@ sCorrect.lvmfit2 <- function(object, ...){
 `sCorrect<-.lme` <- function(x, ..., value){
     x$sCorrect <- sCorrect(x, ..., adjust.Omega = value, adjust.n = value)
     class(x) <- append("lme2",class(x))
-    
     return(x)
 }    
-
+## * sCorrect<-.lme2
+#' @rdname sCorrect
+#' @export
+`sCorrect<-.lme2` <- function(x, ..., value){
+    x$sCorrect <- sCorrect(x, ..., adjust.Omega = value, adjust.n = value)
+    return(x)
+}    
 ## * sCorrect<-.lvmfit
 #' @rdname sCorrect
 #' @export
 `sCorrect<-.lvmfit` <- function(x, ..., value){
-    x$sCorrect <-  try(sCorrect(x, ..., adjust.Omega = value, adjust.n = value), silent = TRUE)
-    if(value == TRUE && inherits(x$sCorrect,"try-error")){
-        warn <- x$sCorrect
-        x$sCorrect <- sCorrect(x, ..., adjust.Omega = value, adjust.n = FALSE)
-        attr(x$sCorrect,"warning") <- warn
-        warning("sCorrect failed and has been re-run setting the argument \'adjust.n\' to FALSE \n",
-                "see the attribute \"warning\" of object$sCorrect for the error message \n")
+    dots <- list(...)
+    safeMode <- dots$safeMode
+    dots[["safeMode"]] <- NULL
+    
+    if(identical(safeMode,TRUE)){
+        x$sCorrect <-  try(do.call(sCorrect,
+                                   args = c(list(x, adjust.Omega = value, adjust.n = value),
+                                            dots) ), silent = TRUE)
+        if(value == TRUE && inherits(x$sCorrect,"try-error")){
+            warn <- x$sCorrect
+            x$sCorrect <- do.call(sCorrect,
+                                  args = c(list(x, adjust.Omega = value, adjust.n = FALSE),
+                                           dots) )
+            attr(x$sCorrect,"warning") <- warn
+            warning("sCorrect failed and has been re-run setting the argument \'adjust.n\' to FALSE \n",
+                    "see the attribute \"warning\" of object$sCorrect for the error message \n")
+        }
+    }else{
+        x$sCorrect <-  do.call(sCorrect,
+                               args = c(list(x, adjust.Omega = value, adjust.n = value),
+                                        dots) )
     }
     class(x) <- append("lvmfit2",class(x))
 
@@ -757,40 +803,11 @@ sCorrect.lvmfit2 <- function(object, ...){
 #' @rdname sCorrect
 #' @export
 `sCorrect<-.lvmfit2` <- function(x, ..., value){
-
-    class(x) <- setdiff(class(x),"lvmfit2")
     x$sCorrect <- sCorrect(x, ..., adjust.Omega = value, adjust.n = value)
-    class(x) <- append("lvmfit2",class(x))
-    
     return(x)
 }
 
-## * .calcResidualsLVM
-.calcResidualsLVM <- function(data, dMoments, n.latent, name.endogenous){
 
-    ## ** fitted value
-    object.fitted <- dMoments$value$nu.XK
-    if(n.latent>0){
-        object.fitted <- object.fitted + dMoments$value$alpha.XGamma.iIB %*% dMoments$value$Lambda
-    }
-
-    ## ** residuals
-    out <- data[, name.endogenous] - object.fitted
-    
-    ## ** export
-    return(as.matrix(out))
-
-}
-
-## * .calcOmegaLVM
-.calcOmegaLVM <- function(dMoments, n.latent){
-    if(n.latent>0){
-        Omega <- dMoments$value$tLambda.tiIB.Psi.iIB %*% dMoments$value$Lambda + dMoments$value$Sigma
-    }else{
-        Omega <- dMoments$value$Sigma
-    }
-    return(Omega)
-}
 
 ##----------------------------------------------------------------------
 ### sCorrect.R ends here
