@@ -3,9 +3,9 @@
 ## Author: Brice Ozenne
 ## Created: mar  6 2018 (10:42) 
 ## Version: 
-## Last-Updated: mar 15 2018 (18:11) 
+## Last-Updated: apr 10 2018 (09:34) 
 ##           By: Brice Ozenne
-##     Update #: 38
+##     Update #: 59
 ##----------------------------------------------------------------------
 ## 
 ### Commentary: 
@@ -22,45 +22,52 @@ if(FALSE){ ## already called in test-all.R
     library(lavaSearch2)
 }
 library(data.table)
-library(lava.tobit)
 library(nlme)
 lava.options(symbols = c("~","~~"))
-context("sCorrect: warnings and errors for invalid objects/arguments")
+context("sCorrect (warnings and errors for invalid objects/arguments)")
 
 ## * Simulation
 n <- 100
 m.sim <- lvm(Y~X1+X2,G~1)
 categorical(m.sim,K=3,label=c("a","b","c")) <- ~G+X2
 set.seed(10)
-d <- sim(m.sim,n,latent=FALSE)
-
+d <- lava::sim(m.sim,n,latent=FALSE)
 
 ## * sCorrect for lvm objects
 
 ## ** error for multigroup lvm
 ## check in sCorrect.R 
-e <- estimate(list(lvm(Y~X1),lvm(Y~X1),lvm(Y~X1)), data = split(d,d$G))
+suppressWarnings(e <- estimate(list(lvm(Y~X1),lvm(Y~X1),lvm(Y~X1)), data = split(d,d$G)))
 test_that("error for multigroup models", {
     expect_error(sCorrect(e))
 })
 
 ## ** error for tobit lvm
 ## check in sCorrect.R
-e <- estimate(lvm(G~X1), data = d)
-test_that("error for tobit models", {
-    expect_error(sCorrect(e))
-})
+if(require(lava.tobit)){
+    e <- estimate(lvm(G~X1), data = d)
+    test_that("error for tobit models", {
+        expect_error(sCorrect(e))
+    })
+}
 
 ## ** error for lvm with transform variables
 ## check in sCorrect.R
 m <- lvm(Y~X1)
 transform(m,Id~X1) <- function(x){1:NROW(x)}
-d.tempo <- sim(m, n)
+d.tempo <- lava::sim(m, n)
 e <- estimate(m, data = d.tempo)
 test_that("error when using transform", {
     expect_error(sCorrect(e))
 })
 
+## ** error for lvm with cluster
+m <- lvm(Y~X1)
+e <- estimate(m, data = d, cluster = 1:NROW(d))
+test_that("error when using cluster", {
+    expect_error(sCorrect(e))
+})
+
 ## * sCorrect for nlme objects
 
 ## ** warning for the satterthwaite approx. with REML
@@ -71,8 +78,7 @@ test_that("warning when using nlme with REML and Satterthwaite", {
     expect_warning(sCorrect(e, score = FALSE, df = FALSE, trace = 0) <- FALSE)
 })
 
-
-## ** error for the small sample correction estimated with REML
+## ** warning for the small sample correction estimated with REML
 ## check in sCorrect.R
 e <- gls(Y~X1, data = d, correlation = corCompSymm(form =~1|G))
 test_that("warning when using nlme with REML and small sample correction", {
@@ -100,8 +106,6 @@ test_that("error when using nlme with non standard variance", {
     expect_error(sCorrect(e, cluster = 1:NROW(d), trace = 0))
 })
 
-##----------------------------------------------------------------------
-### test1-sCorrect-validObjects.R ends here
 
 ## * sCorrect with data.table
 
@@ -110,3 +114,7 @@ test_that("ok for data.table objects", {
     sCorrect(e) <- FALSE
     sCorrect(e) <- TRUE
 })
+
+##----------------------------------------------------------------------
+### test1-sCorrect-validObjects.R ends here
+
